// Bookmark Backup 0.4.1
// By Pike

// Contributor(s):
// Nickolay Ponomarev (Multiple file backup code)

var BookmarkBackupSettings = {

	backupGroups: {
		history: ['history.dat', 'history.txt'],
		IconCache: ['IconCache.dat'],
		passwords: ['signons.txt', 'key3.db', '*.s'],
		cookies: ['cookies.txt'],
		perms: ['hostperm.1'],
		prefs: ['prefs.js'],
		macros: ['macros'],
		fileTypes: ['mimeTypes.rdf'],
		security: ['cert8.db', 'secmod.db'],
		hotlist: ['opera.adr'],
		newsfox: ['newsfox/*.xml', 'feeds.xml'],
		userJs: ['user.js'],
		userCss: ['chrome/userChrome.css'],
		userContent: ['chrome/userContent.css', 'chrome/adblock.css'],
		userAccel: ['accel.cfg'],
		userMenus: ['menus.cfg'],
		userGestures: ['gestures.xml'],
		userSearch: ['search.xml']

	},
	
	onLoad: function()
	{
		// Setup object properties
		this.localeStrings = document.getElementById('bookmarkBackupStrings');
		this.prefBranch = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefService).getBranch('bookmarkbackup.');
		this.whereField = document.getElementById('where');
		this.locationField = document.getElementById('location');
		this.browseButton = document.getElementById('browse');
		this.otherFilesField = document.getElementById('otherFiles');
		
		// Add tooltips
		for (var i in this.backupGroups)
		{
			var tooltipText = this.localeStrings.getFormattedString('backsUp', [this.backupGroups[i].join(', ')]);
			document.getElementById(i).setAttribute('tooltiptext', tooltipText);
		}
		
		// Backup location
		var location = null;
		try
		{
			location = this.prefBranch.getComplexValue('location', Components.interfaces.nsILocalFile).path;
		}
		catch (err) {}
		if (location)
		{
			this.whereField.selectedItem = document.getElementById('custom');
			this.locationField.value = location;
		}
		else
		{
			this.whereField.selectedItem = document.getElementById('default');
		}
		
		// Backup list
		var backupList = null;
		try
		{
			backupList = this.prefBranch.getComplexValue('backupList', Components.interfaces.nsISupportsString).data;
		}
		catch (err) {}
		if (backupList)
		{
			var backupFileNames = backupList.split('|').sort();
			var toDelete = new Array();
			for (var i in this.backupGroups)
			{
				if (this.containsArray(backupFileNames, this.backupGroups[i].sort()))
				{
					document.getElementById(i).checked = true;
					toDelete = toDelete.concat(this.backupGroups[i]);
				}
			}
			backupFileNames = this.deleteIntersection(backupFileNames, toDelete.sort());
			this.otherFilesField.value = backupFileNames.join('|');
		}
	},
	
	onAccept: function()
	{
		// Backup location
		var location = this.trimString(this.locationField.value);
		if (this.whereField.selectedItem.id == 'custom' && location)
		{
			var customBackupsDir = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
			customBackupsDir.initWithPath(location);
			this.prefBranch.setComplexValue('location', Components.interfaces.nsILocalFile, customBackupsDir);
		}
		else
		{
			try
			{
				this.prefBranch.clearUserPref('location');
			}
			catch(err) {}
		}
		
		// Backup list
		var newFileNames = new Array();
		var i;
		for (i in this.backupGroups)
		{
			if (document.getElementById(i).checked)
			{
				newFileNames = newFileNames.concat(this.backupGroups[i]);
			}
		}
		var otherFiles = this.otherFilesField.value.split('|');
		for (i = 0; i < otherFiles.length; i++)
		{
			otherFiles[i] = this.trimString(otherFiles[i]);
			if (otherFiles[i])
			{
				newFileNames.push(otherFiles[i]);
			}
		}
		var backupList = this.deleteDuplicates(newFileNames.sort()).join('|');
		if (backupList)
		{
			var backupListStr = Components.classes['@mozilla.org/supports-string;1'].createInstance(Components.interfaces.nsISupportsString);
			backupListStr.data = backupList;
			this.prefBranch.setComplexValue('backupList', Components.interfaces.nsISupportsString, backupListStr);
		}
		else
		{
			try
			{
				this.prefBranch.clearUserPref('backupList');
			}
			catch(err) {}
		}
	},
	
	onSelect: function(selected)
	{
		if (selected.id == 'default')
		{
			this.locationField.disabled = true;
			this.browseButton.disabled = true;
		}
		else
		{
			this.locationField.disabled = false;
			this.browseButton.disabled = false;
		}
	},
	
	browse: function()
	{
		var filePicker = Components.classes['@mozilla.org/filepicker;1'].createInstance(Components.interfaces.nsIFilePicker);
		filePicker.init(window, this.localeStrings.getString('selectBackupFolder'), filePicker.modeGetFolder);
		try
		{
			var location = this.trimString(this.locationField.value);
			if (location)
			{
				var currentLocation = Components.classes['@mozilla.org/file/local;1'].createInstance(Components.interfaces.nsILocalFile);
				currentLocation.initWithPath(location);
				filePicker.displayDirectory = currentLocation;
			}
		}
		catch (err) {}
		filePicker.appendFilters(filePicker.filterAll);
		var result = filePicker.show();
		if (result == filePicker.returnOK)
		{
			this.locationField.value = filePicker.file.path;
		}
	},
	
	selectAll: function()
	{
		for (var i in this.backupGroups)
		{
			document.getElementById(i).checked = true;
		}
	},
	
	selectNone: function()
	{
		for (var i in this.backupGroups)
		{
			document.getElementById(i).checked = false;
		}
	},
	
	containsArray: function(a, b)
	// This function is *not* general-purpose, both the a and b parameters *must* be pre-sorted arrays
	{
		var bPos = 0;
		for (var i = 0; i < a.length && bPos < b.length; i++)
		{
			if (a[i] == b[bPos])
			{
				bPos++;
			}
		}
		if (bPos == b.length)
		{
			return true;
		}
		else
		{
			return false;
		}
	},
	
	deleteIntersection: function(a, b)
	// This function is *not* general-purpose, both the a and b parameters *must* be pre-sorted arrays
	{
		var outArray = new Array();
		var bPos = 0;
		for (var i = 0; i < a.length; i++)
		{
			if (bPos < b.length && a[i] == b[bPos])
			{
				bPos++;
			}
			else
			{
				outArray.push(a[i]);
			}
		}
		return outArray;
	},
	
	deleteDuplicates: function(input)
	// This function is *not* general-purpose, the input parameter *must* be a pre-sorted array
	{
		var outArray = new Array();
		var lastValue;
		for (var i = 0; i < input.length; i++)
		{
			if (!i || input[i] != lastValue)
			{
				outArray.push(input[i]);
				lastValue = input[i];
			}
		}
		return outArray;
	},
	
	trimString: function(string)
	{
		return string.replace(/(^\s+)|(\s+$)/g, '');
	}
};
